@@ -1,18 +1,19 @@
 import asyncio
+import re
 
 from bs4 import BeautifulSoup
 from colorama import Fore
 from yarl import URL
 
 from ..base_functions import log, logger, make_title_safe, ssl_context
-from ..data_classes import CascadeItem
+from ..data_classes import AuthData, CascadeItem
 
 
-class ThotsbayCrawler():
-    def __init__(self, *, include_id=False, username=None, password=None, scraping_mapper, session):
+class ThotsbayCrawler:
+    def __init__(self, *, include_id=False, auth: AuthData = None, scraping_mapper, session, separate_posts=False):
         self.include_id = include_id
-        self.username = username
-        self.password = password
+        self.separate_posts = separate_posts
+        self.username, self.password = (auth.username, auth.password) if auth else (None, None)
         self.scraping_mapper = scraping_mapper
         self.session = session
         self.lock = 0
@@ -61,7 +62,7 @@ class ThotsbayCrawler():
         except Exception:
             await log("Error handling " + str(url))
             return
-        await Cascade.append_title(title)
+        # await Cascade.append_title(title)
 
         await log("Finished scrape of " + str(url), Fore.WHITE)
         return Cascade
@@ -93,14 +94,16 @@ class ThotsbayCrawler():
                 post_number = str(url).split("post-")
                 post_number = int(post_number[-1].strip("/")) if len(post_number) == 2 else None
 
-
                 posts = soup.select("div[class='message-main uix_messageContent js-quickEditTarget']")
                 for post in posts:
+                    post_num_str = post.select_one("li[class=u-concealed] a").get('href').split('/')[-1]
+                    post_num_int = int(post_num_str.split('post-')[-1])
                     if post_number:
-                        post_num_int = post.select_one("li[class=u-concealed] a")
-                        post_num_int = int(post_num_int.get('href').split('post-')[-1])
                         if post_number > post_num_int:
                             continue
+
+                    temp_title = title+"/"+post_num_str if self.separate_posts else title
+
                     for elem in post.find_all('blockquote'):
                         elem.decompose()
                     post_content = post.select_one("div[class=bbWrapper]")
@@ -115,7 +118,7 @@ class ThotsbayCrawler():
                             link = "https:" + link
                         elif link.startswith('/'):
                             link = domain / link[1:]
-                        content_links.append(URL(link))
+                        content_links.append([URL(link), temp_title])
 
                     links = post.select("div[class='bbImageWrapper js-lbImage']")
                     for link in links:
@@ -124,7 +127,7 @@ class ThotsbayCrawler():
                             link = link[:-1]
                         if link.startswith('/'):
                             link = domain / link[1:]
-                        content_links.append(URL(link))
+                        content_links.append([URL(link), temp_title])
 
                     links = post.select("div[class='bbImageWrapper lazyload js-lbImage']")
                     for link in links:
@@ -135,7 +138,7 @@ class ThotsbayCrawler():
                             link = "https:" + link
                         elif link.startswith('/'):
                             link = domain / link[1:]
-                        content_links.append(URL(link))
+                        content_links.append([URL(link), temp_title])
 
                     links = post.select("video source")
                     for link in links:
@@ -146,7 +149,7 @@ class ThotsbayCrawler():
                             link = "https:" + link
                         elif link.startswith('/'):
                             link = domain / link[1:]
-                        content_links.append(URL(link))
+                        content_links.append([URL(link), temp_title])
 
                     attachments_block = post.select_one("section[class=message-attachments]")
                     links = attachments_block.select("a[class='file-preview js-lbImage']") if attachments_block else []
@@ -156,21 +159,44 @@ class ThotsbayCrawler():
                             link = link[:-1]
                         if link.startswith('/'):
                             link = domain / link[1:]
-                        await Cascade.add_to_album(url.host, "Attachments", URL(link), url)
+                        in_prog_title = temp_title + "/Attachments"
+                        await Cascade.add_to_album(url.host, in_prog_title, URL(link), url)
 
-                forum_direct_urls = [x for x in content_links if url.host in x.host]
+                    # Gyfcat / Redgifs / Imgur
+                    # NOTE IMGUR SUPPORT IS NOT WORKING AS OF NOW
+                    links = post_content.select("span[data-s9e-mediaembed-iframe]")
+                    for link in links:
+                        embed_data = link.get("data-s9e-mediaembed-iframe")
+                        # Replace gyfcat/redgifs/imgur embed url escape chars
+                        embed_data = embed_data.replace("\/\/", "https://www.")
+                        embed_data = embed_data.replace("\\", "")
+
+                        # extract gyfcat/redgifs/imgur url from embed data
+                        embed_url = re.search(
+                            "https?:\/\/(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,4}\\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)", embed_data)
+                        if embed_url:
+                            embed_url = URL(embed_url.group(
+                                0).replace("www.", ""))
+                            content_links.append([embed_url, temp_title])
+
+                forum_direct_urls = [x for x in content_links if url.host in x[0].host]
                 content_links = [x for x in content_links if x not in forum_direct_urls]
-                for link in forum_direct_urls:
+                for link_title_bundle in forum_direct_urls:
+                    link = link_title_bundle[0]
+                    temp_title = link_title_bundle[1]
+                    in_prog_title = temp_title + "/Attachments"
                     if str(link).endswith("/"):
                         link = URL(str(link)[:-1])
                     if 'attachments' in link.parts:
-                        await Cascade.add_to_album(url.host, "Attachments", link, url)
+                        await Cascade.add_to_album(url.host, in_prog_title, link, url)
                     elif 'data' in link.parts:
-                        await Cascade.add_to_album(url.host, "Attachments", link, url)
+                        await Cascade.add_to_album(url.host, in_prog_title, link, url)
 
                 tasks = []
-                for link in content_links:
-                    tasks.append(self.scraping_mapper.map_url(link, title))
+                for link_title_bundle in content_links:
+                    link = link_title_bundle[0]
+                    temp_title = link_title_bundle[1]
+                    tasks.append(self.scraping_mapper.map_url(link, temp_title))
                 await asyncio.gather(*tasks)
 
                 next_page = soup.select_one('a[class="pageNav-jump pageNav-jump--next"]')
@@ -185,4 +211,4 @@ class ThotsbayCrawler():
 
         except Exception as e:
             logger.debug("Error encountered while handling %s", str(url), exc_info=True)
-            logger.debug(e)
+            logger.debug(e)
\ No newline at end of file
