@@ -1,5 +1,7 @@
+import asyncio
 from dataclasses import dataclass
 from typing import Dict, List, Optional, Tuple
+from random import gauss
 
 import aiohttp
 from yarl import URL
@@ -19,6 +21,23 @@ class AlbumItem:
         self.title = new_title
 
 
+@dataclass
+class FileLock:
+    locked_files = []
+
+    async def check_lock(self, filename):
+        await asyncio.sleep(gauss(0, 2))
+        if filename.lower() in self.locked_files:
+            return True
+        return False
+
+    async def add_lock(self, filename):
+        self.locked_files.append(filename.lower())
+
+    async def remove_lock(self, filename):
+        self.locked_files.remove(filename.lower())
+
+
 @dataclass
 class DomainItem:
     domain: str
@@ -28,7 +47,8 @@ class DomainItem:
         if title in self.albums.keys():
             await self.albums[title].add_link_pair(link, referral)
         else:
-            self.albums[title] = AlbumItem(title=title, link_pairs=[(link, referral)])
+            self.albums[title] = AlbumItem(
+                title=title, link_pairs=[(link, referral)])
 
     async def add_album(self, title: str, album: AlbumItem):
         if title in self.albums.keys():
@@ -66,7 +86,8 @@ class CascadeItem:
         if domain in self.domains.keys():
             await self.domains[domain].add_to_album(title, link, referral)
         else:
-            self.domains[domain] = DomainItem(domain, {title: AlbumItem(title, [(link, referral)])})
+            self.domains[domain] = DomainItem(
+                domain, {title: AlbumItem(title, [(link, referral)])})
 
     async def add_album(self, domain: str, title: str, album: AlbumItem):
         if domain in self.domains.keys():
@@ -111,3 +132,10 @@ class CascadeItem:
                         check.append(url)
                         allowed.append(pair)
                 album.link_pairs = allowed
+
+
+@dataclass
+class AuthData:
+    """Class for keeping username and password"""
+    username: str
+    password: str
