@@ -1,7 +1,6 @@
 import asyncio
 import logging
 import multiprocessing
-import sqlite3
 import time
 import traceback
 from functools import wraps
@@ -15,9 +14,9 @@ from colorama import Fore
 from tqdm import tqdm
 from yarl import URL
 
-from .base_functions import FILE_FORMATS, MAX_FILENAME_LENGTH, log, logger, sanitize, sql_check_existing, \
-    sql_insert_file, sql_update_file, ssl_context, user_agent
-from .data_classes import AlbumItem, CascadeItem
+from .base_functions import FILE_FORMATS, MAX_FILENAME_LENGTH, log, logger, sanitize, ssl_context, user_agent
+from .sql_helper import SQLHelper
+from .data_classes import AlbumItem, CascadeItem, FileLock
 
 
 class FailureException(Exception):
@@ -63,7 +62,8 @@ async def throttle(self, url: URL) -> None:
 
         remaining = delay - elapsed + 0.25
 
-        log_string = '\nDelaying request to %s for %.2f seconds.' % (host, remaining)
+        log_string = '\nDelaying request to %s for %.2f seconds.' % (
+            host, remaining)
         logger.debug(log_string)
         await asyncio.sleep(remaining)
 
@@ -71,14 +71,14 @@ async def throttle(self, url: URL) -> None:
 class Downloader:
     def __init__(self, album_obj: AlbumItem, cookie_jar, folder: Path, title: str, attempts: int,
                  disable_attempt_limit: bool, max_workers: int, exclude_videos: bool, exclude_images: bool,
-                 exclude_audio: bool, exclude_other: bool, connection: sqlite3.Connection, cursor: sqlite3.Cursor):
+                 exclude_audio: bool, exclude_other: bool, SQL_helper: SQLHelper):
         self.album_obj = album_obj
         self.cookie_jar = cookie_jar
         self.folder = folder
         self.title = title
 
-        self.connection = connection
-        self.cursor = cursor
+        self.SQL_helper = SQL_helper
+        self.FileLocker = FileLock()
 
         self.attempts = attempts
         self.current_attempt = 0
@@ -118,9 +118,32 @@ class Downloader:
                     async with session.get(url, headers=headers, ssl=ssl_context, raise_for_status=True) as resp:
                         filename = resp.content_disposition.filename
                         filename = await sanitize(filename)
-                        del resp
-                        if (self.folder / self.title / filename).exists():
+                        total_size = int(resp.headers.get('Content-Length', str(0)))
+                else:
+                    async with session.get(url, headers=headers, ssl=ssl_context, raise_for_status=True) as resp:
+                        total_size = int(resp.headers.get('Content-Length', str(0)))
+
+                # Make suffix always lower case
+                filename.replace('.'+filename.split('.')[-1], ext)
+
+                # Check DB for already downloaded
+                if await self.SQL_helper.sql_check_existing(filename, total_size):
+                    logger.debug("%s Already Downloaded" % filename)
+                    return
+
+                complete_file = (self.folder / self.title / filename)
+                if await self.FileLocker.check_lock(filename) or complete_file.exists():
+                    if complete_file.exists():
+                        if complete_file.stat().st_size == total_size:
+                            await self.SQL_helper.sql_update_file(filename, complete_file.stat().st_size, 1)
+                            logger.debug("%s Already Downloaded" % filename)
                             return
+                        else:
+                            await log("\nAnother file already exists with filename: " + filename)
+                            return
+                    await log("\nAnother file already downloading with filename: " + filename)
+                    return
+                await self.FileLocker.add_lock(filename)
 
                 # Skip based on CLI arg.
                 if self.exclude_videos:
@@ -142,7 +165,8 @@ class Downloader:
 
                 resume_point = 0
                 complete_file = (self.folder / self.title / filename)
-                temp_file = complete_file.with_suffix(complete_file.suffix + '.part')
+                temp_file = complete_file.with_suffix(
+                    complete_file.suffix + '.part')
 
                 if temp_file.exists():
                     resume_point = temp_file.stat().st_size
@@ -151,17 +175,14 @@ class Downloader:
                 async with session.get(url, headers=headers, ssl=ssl_context, raise_for_status=True) as resp:
                     content_type = resp.headers.get('Content-Type')
                     if 'text' in content_type.lower() or 'html' in content_type.lower():
-                        logger.debug(f"Server for %s is either down or the file no longer exists" % str(url))
+                        logger.debug(
+                            f"Server for %s is either down or the file no longer exists" % str(url))
                         return
                     total = int(resp.headers.get('Content-Length', str(0))) + resume_point
 
-                    if await sql_check_existing(self.cursor, filename, total):
-                        logger.debug("%s Already Downloaded" % filename)
-                        return
-
                     (self.folder / self.title).mkdir(parents=True, exist_ok=True)
 
-                    await sql_insert_file(self.connection, self.cursor, filename, total, 0)
+                    await self.SQL_helper.sql_insert_file(filename, total, 0)
 
                     with tqdm(
                             total=total, unit_scale=True,
@@ -173,30 +194,41 @@ class Downloader:
                                 await f.write(chunk)
                                 progress.update(len(chunk))
             resp.close()
-            await self.rename_file(filename)
+            await self.rename_file(filename, total_size)
+            await self.FileLocker.remove_lock(filename)
         except (aiohttp.client_exceptions.ClientPayloadError, aiohttp.client_exceptions.ClientOSError,
                 aiohttp.client_exceptions.ServerDisconnectedError, asyncio.TimeoutError,
                 aiohttp.client_exceptions.ClientResponseError, FailureException) as e:
+            try:
+                await self.FileLocker.remove_lock(filename)
+            except:
+                pass
+
             try:
                 if 400 <= e.code < 500:
-                    logger.debug("We ran into a 400 level error: %s" % str(e.code))
+                    logger.debug(
+                        "We ran into a 400 level error: %s" % str(e.code))
                     return
                 resp.close()
             except:
                 pass
+
             raise FailureException(e)
 
-    async def rename_file(self, filename: str) -> None:
+    async def rename_file(self, filename: str, total_size: int) -> None:
         """Rename complete file."""
         complete_file = (self.folder / self.title / filename)
         temp_file = complete_file.with_suffix(complete_file.suffix + '.part')
-        if complete_file.exists():
-            logger.debug(str(self.folder / self.title / filename) + " Already Exists")
-            await aiofiles.os.remove(temp_file)
+        if temp_file.stat().st_size == total_size:
+            if complete_file.exists():
+                logger.debug(str(self.folder / self.title / filename) + " Already Exists")
+                await aiofiles.os.remove(temp_file)
+            else:
+                temp_file.rename(complete_file)
+            await self.SQL_helper.sql_update_file(filename, complete_file.stat().st_size, 1)
+            logger.debug("Finished " + filename)
         else:
-            temp_file.rename(complete_file)
-        await sql_update_file(self.connection, self.cursor, filename, complete_file.stat().st_size, 1)
-        logger.debug("Finished " + filename)
+            await log("File size doesn't match expected size: " + temp_file.name)
 
     async def download_and_store(
             self,
@@ -215,18 +247,13 @@ class Downloader:
             fileext = filename.split('.')[-1]
             filename = filename[:MAX_FILENAME_LENGTH] + '.' + fileext
 
-        complete_file = (self.folder / self.title / filename)
-        if complete_file.exists():
-            await sql_update_file(self.connection, self.cursor, filename, complete_file.stat().st_size, 1)
-            logger.debug(str(complete_file) + " Already Exists")
-        else:
-            logger.debug("Working on " + str(url))
-            try:
-                await self.download_file(url, referral=referral, filename=filename,
-                                         session=session, show_progress=show_progress)
-            except Exception:
-                logger.debug(traceback.format_exc())
-                await log(f"\nError attempting {filename}: See downloader.log for details\n", Fore.WHITE)
+        logger.debug("Working on " + str(url))
+        try:
+            await self.download_file(url, referral=referral, filename=filename,
+                                     session=session, show_progress=show_progress)
+        except Exception:
+            logger.debug(traceback.format_exc())
+            await log(f"\nError attempting {filename}: See downloader.log for details\n", Fore.WHITE)
 
     async def download_all(
             self,
@@ -244,11 +271,12 @@ class Downloader:
         """Download the content of all links and save them as files."""
         async with aiohttp.ClientSession(cookie_jar=self.cookie_jar) as session:
             await self.download_all(self.album_obj, session, show_progress=show_progress)
-        self.connection.commit()
+        self.SQL_helper.conn.commit()
 
 
-def get_downloaders(Cascade: CascadeItem, folder: Path, attempts: int, disable_attempt_limit: bool, threads: int, exclude_videos: bool,
-                    exclude_images: bool, exclude_audio: bool, exclude_other: bool, connection:sqlite3.Connection, cursor: sqlite3.Cursor) -> List[Downloader]:
+async def get_downloaders(Cascade: CascadeItem, folder: Path, attempts: int, disable_attempt_limit: bool, threads: int,
+                    exclude_videos: bool, exclude_images: bool, exclude_audio: bool, exclude_other: bool,
+                    SQL_helper: SQLHelper) -> List[Downloader]:
     """Get a list of downloaders for each supported type of URLs.
     We shouldn't just assume that each URL will have the same netloc as
     the first one, so we need to classify them one by one, sort them to
@@ -269,6 +297,6 @@ def get_downloaders(Cascade: CascadeItem, folder: Path, attempts: int, disable_a
                                     attempts=attempts, disable_attempt_limit=disable_attempt_limit,
                                     max_workers=max_workers, exclude_videos=exclude_videos,
                                     exclude_images=exclude_images, exclude_audio=exclude_audio,
-                                    exclude_other=exclude_other, connection=connection, cursor=cursor)
+                                    exclude_other=exclude_other, SQL_helper=SQL_helper)
             downloaders.append(downloader)
     return downloaders
